package com.muyu.marketing.controller;


import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.model.ActivityTeamInfoAddModel;
import com.muyu.marketing.domain.model.ActivityTeamInfoListModel;
import com.muyu.marketing.domain.req.ActivityProductUpdReq;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.ActivityTeamInfoDetailResp;
import com.muyu.marketing.domain.resp.TeamInfoListResp;
import com.muyu.marketing.service.ActivityTeamInfoService;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 营销团购活动控制层
 *
 * @author DongZeLiang
 * @date 2024-11-20 14:25
 */
@RestController
@RequestMapping("/team")
public class ActivityTeamController {

    @Autowired
    private ActivityTeamInfoService activityTeamInfoService;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    /**
     * 查询营销团购活动列表
     * @param teamInfoListReq 活动查询入参
     * @return 活动响应结果
     */
    @PostMapping("/list")
    public Result<TableDataInfo<TeamInfoListResp>> list(@RequestBody TeamInfoListReq teamInfoListReq) {
        // 查询活动列表
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = activityTeamInfoService.query(teamInfoListReq.buildQueryModel());
        // 转换成响应结果
        List<TeamInfoListResp> respList = tableDataInfo.getRows().stream().map(TeamInfoListResp::listModelBuild).toList();
        // 返回结果
        return Result.success(
                new TableDataInfo<>(){{
                    setRows(respList);
                    setTotal(tableDataInfo.getTotal());
                }}
        );
    }

    /**
     * 添加营销团购活动
     */
    @PostMapping("insert")
    public Result<String> insert(@RequestBody ActivityTeamInfoSaveReq activityTeamInfoSaveReq) {
        // 保存活动信息
        activityTeamInfoService.save(ActivityTeamInfoAddModel.addBuild(activityTeamInfoSaveReq));
        return Result.success();
    }

    /**
     * 获取营销团购活动信息
     */
    @GetMapping("/{teamId}")
    public Result<ActivityTeamInfoDetailResp> getTeamInfo(@PathVariable("teamId") Long teamId){
        return Result.success(activityTeamInfoService.getActivityByTeamId(teamId));
    }

    /**
     * 营销活动的修改
     */
    @PutMapping("update")
    public Result<String> update(@RequestBody ActivityProductUpdReq activityProductUpdReq){
        activityTeamInfoService.edit(activityProductUpdReq);
        return Result.success();
    }


    /**
     * 团购活动列表
     */
    @GetMapping("/teamList")
    public Result<TableDataInfo<ActivityTeamInfo>> teamList(){
        List<ActivityTeamInfo> list = activityTeamInfoService.list();
        TableDataInfo<ActivityTeamInfo> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setRows(list);
        tableDataInfo.setTotal(list.size());
        return Result.success(tableDataInfo);
    }
}
