package com.muyu.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.utils.bean.BeanUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.ActivityProductUpdReq;
import com.muyu.marketing.domain.resp.ActivityProductUpdResp;
import com.muyu.marketing.domain.resp.ActivityTeamInfoDetailResp;
import com.muyu.marketing.domain.resp.ActivityTeamProductSkuResp;
import com.muyu.marketing.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.service.ActivityTeamInfoService;
import com.muyu.marketing.service.ActivityTeamOpenInfoService;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;


@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo>
        implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

//    @Autowired
//    private ActivityTeamInfoMapper activityTeamInfoMapper;


    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query(ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {
        // 查询条件
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        // 关键字查询
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()), ActivityTeamInfo::getName, activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()), ActivityTeamInfo::getStatus, activityTeamInfoListQueryModel.getStatus());

        /**
         * Object<T> -> 创建对象的时候进行的占用
         * <T> Result<T> 以方法返回值为占用
         */
        // 分页查询
        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
        // 查询数据列表
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        // 数据转换
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                        (activityTeamInfoListModelBuilder) -> {
                            //获取折扣价
                            TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                            //根据活动id获取库存模型
                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                            //获取已开团和已参团人数
                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());
                            //封装活动详情
                            return activityTeamInfoListModelBuilder
                                    .openTeamNumber(teamOpenTypeNumber)
                                    .addTeamNumber(teamInTypeNumber)
                                    .attendNumber(teamOpenTypeNumber + teamInTypeNumber)
                                    .teamPrice(discountPrice.getTeamPrice())
                                    .productPrice(discountPrice.getProductPrice())
                                    .teamStock(teamProductStockModel.getTeamStock())
                                    .remainStock(teamProductStockModel.getRemainStock())
                                    .build();
                        })).toList();
        // 封装分页数据
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);
        return tableDataInfo;
    }
    /**
     * 通过添加模型，去进行商品拼团活动的操作
     *
     * @param activityTeamInfoAddModel 团购添加模型
     */
    @Override
    public void save(ActivityTeamInfoAddModel activityTeamInfoAddModel) {
        // 添加团购活动
        boolean save = this.save(ActivityTeamInfo.addModelBuild(activityTeamInfoAddModel));
        //判断是否添加成功
        if (!save) {
            throw new ServiceException(
                    StringUtils.format("团购活动添加失败：[{}]", activityTeamInfoAddModel)
            );
        }
        //拼团活动添加后添加商品SKU
        this.activityTeamProductSkuInfoService.batchSave(
                activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList()
        );
    }

    /**
     * 根据id查询活动详情
     *
     * @param teamId
     * @return
     */
    @Override
    public ActivityTeamInfoDetailResp getActivityByTeamId(Long teamId) {
        //根据id查询活动详情
        ActivityTeamInfo activityTeamInfo = getById(teamId);
        //封装活动详情
        ActivityTeamInfoDetailResp activityTeamInfoDetailResp = new ActivityTeamInfoDetailResp();
        //根据活动id查询活动商品SKU集合
        List<ActivityProductSkuListModel> teamProductSkuInfoList = activityTeamProductSkuInfoService.getTeamProductSkuInfoListByTeamId(teamId);
        if (teamProductSkuInfoList != null) {
            //定义一个空集合
            ArrayList<ActivityTeamProductSkuResp> activityProductSkuList = new ArrayList<>();
            for (ActivityProductSkuListModel activityProductSkuListModel : teamProductSkuInfoList) {
                Long id = activityProductSkuListModel.getId();
                //根据id查询
                ActivityTeamProductSkuInfo activityTeamProductSkuInfo = activityTeamProductSkuInfoService.getById(id);
                //添加到集合中
                activityProductSkuList.add(
                        ActivityTeamProductSkuResp.builder()
                                .ProductId(activityTeamProductSkuInfo.getProductId())
                                .sku(activityTeamProductSkuInfo.getProductSku())
                                .teamPrice(activityTeamProductSkuInfo.getTeamPrice())
                                .teamStock(activityTeamProductSkuInfo.getTeamStock())
                                .build()
                );
            }
            //赋值
            activityTeamInfoDetailResp.setActivityTeamProductSkuList(activityProductSkuList);
            //克隆到activityTeamInfo
            BeanUtils.copyBeanProp(activityTeamInfoDetailResp, activityTeamInfo);
        }
        return activityTeamInfoDetailResp;
    }

    /**
     * 修改团购活动信息
     * @param activityProductUpdReq
     */
    @Transactional
    @Override
    public void edit(ActivityProductUpdReq activityProductUpdReq) {
        //获取活动商品SKU
        ActivityProductUpdResp activityProductUpdResp = activityProductUpdReq.getActivityProductUpdResp();
        //获取需要删除的SKU集合
        List<Long> deleteSkuIdList = activityProductUpdResp.getDeleteSkuIdList();
        //判断是否为空
        if (deleteSkuIdList != null){
            activityTeamProductSkuInfoService.deleteBatchByTeamIds(deleteSkuIdList);
        }
        //获取需要修改的SKU集合
        List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuList = activityProductUpdResp.getActivityTeamProductSkuUpdList();
        //判断是否为空
        if (activityTeamProductSkuList != null){
            activityTeamProductSkuList.stream().map(productSkuUpdModel -> {
                return ProductSkuUpdModel.builder()
                        .productId(productSkuUpdModel.getProductId())
                        .teamPrice(productSkuUpdModel.getTeamPrice())
                        .teamStock(productSkuUpdModel.getTeamStock())
                        .build();
            }).toList();
            //批量修改SKU集合
            activityTeamProductSkuInfoService.editBatchActivityTeamProductSkuUpdModelList(activityTeamProductSkuList);
        }

        //获取要添加的SKU集合
        List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddList = activityProductUpdResp.getActivityTeamProductSkuAddList();
        //判断是否为空
        if (activityTeamProductSkuAddList != null){
            activityTeamProductSkuAddList.stream().map(productSkuAddModel -> {
                return ProductSkuAddModel.builder()
                        .productId(productSkuAddModel.getProductId())
                        .teamPrice(productSkuAddModel.getTeamPrice())
                        .teamStock(productSkuAddModel.getTeamStock())
                        .productSku(productSkuAddModel.getProductSku())
                        .build();
            }).toList();
            //批量添加SKU集合
            activityTeamProductSkuInfoService.batchSave(activityTeamProductSkuAddList);

        }
        //修改活动信息
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.builder()
                .id(activityProductUpdReq.getId())
                .name(activityProductUpdReq.getName())
                .projectId(activityProductUpdReq.getProjectId())
                .productImage(activityProductUpdReq.getProductImage())
                .introduction(activityProductUpdReq.getIntroduction())
                .unit(activityProductUpdReq.getUnit())
                .imageList(activityProductUpdReq.getImageList())
                .endTime(activityProductUpdReq.getEndTime())
                .sort(activityProductUpdReq.getSort())
                .content(activityProductUpdReq.getContent())
                .status(activityProductUpdReq.getStatus())
                .strategyType(activityProductUpdReq.getStrategyType())
                .strategyId(activityProductUpdReq.getStrategyId())
                .build();
        //修改拼团活动
        updateById(activityTeamInfo);
    }




}
