package com.muyu.product.service.impl;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.*;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import com.muyu.product.domain.req.ProjectInfoSaveReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.ProjectDetailResp;
import com.muyu.product.domain.resp.ProjectInfoListResp;
import com.muyu.product.mapper.BrandInfoMapper;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.service.*;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.ProjectInfoMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.util.CollectionUtils;

/**
 * 商品信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Component
@Service
public class ProjectInfoServiceImpl extends ServiceImpl<ProjectInfoMapper, ProjectInfo>  implements ProjectInfoService {


    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;

    @Autowired
    private ProjectSkuInfoService projectSkuInfoService;

    @Autowired
    private CategoryInfoService categoryInfoService;

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeInfoService attributeInfoService;
    @Autowired
    private ProjectInfoMapper projectInfoMapper;
    @Autowired
    private RuleInfoMapper ruleInfoMapper;
    @Autowired
    private BrandInfoMapper brandInfoMapper;

    /**
     * 查询商品信息列表
     *
     * @param projectInfo 商品信息
     * @return 商品信息
     */
    @Override
    public List<ProjectInfo> list(ProjectInfo projectInfo) {
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(projectInfo.getName())){
            queryWrapper.like(ProjectInfo::getName, projectInfo.getName());
        }

        if (ObjUtils.notNull(projectInfo.getIntroduction())){
            queryWrapper.eq(ProjectInfo::getIntroduction, projectInfo.getIntroduction());
        }

        if (ObjUtils.notNull(projectInfo.getMianType())){
            queryWrapper.eq(ProjectInfo::getMianType, projectInfo.getMianType());
        }

        if (ObjUtils.notNull(projectInfo.getParentType())){
            queryWrapper.eq(ProjectInfo::getParentType, projectInfo.getParentType());
        }

        if (ObjUtils.notNull(projectInfo.getType())){
            queryWrapper.eq(ProjectInfo::getType, projectInfo.getType());
        }

        if (ObjUtils.notNull(projectInfo.getImage())){
            queryWrapper.eq(ProjectInfo::getImage, projectInfo.getImage());
        }

        if (ObjUtils.notNull(projectInfo.getCarouselImages())){
            queryWrapper.eq(ProjectInfo::getCarouselImages, projectInfo.getCarouselImages());
        }

        if (ObjUtils.notNull(projectInfo.getStatus())){
            queryWrapper.eq(ProjectInfo::getStatus, projectInfo.getStatus());
        }

        if (ObjUtils.notNull(projectInfo.getRuleId())){
            queryWrapper.eq(ProjectInfo::getRuleId, projectInfo.getRuleId());
        }

        if (ObjUtils.notNull(projectInfo.getBrandId())){
            queryWrapper.eq(ProjectInfo::getBrandId, projectInfo.getBrandId());
        }





        return list(queryWrapper);
    }

    /**
     * 保存商品信息
     *
     * @param projectInfoSaveReq 请求对象
     *
     * @return
     */
    @Override
    public boolean save (ProjectInfoSaveReq projectInfoSaveReq) {
        ProjectAddModel projectAddModel1 = projectInfoSaveReq.getProjectAddModel();
        String name = projectAddModel1.getName();
        //根据name查询project_info数据库
        ProjectInfo getProjectInfo=projectInfoMapper.getProjectName(name);
        if (getProjectInfo!=null){
            return false;
        }
        ProjectAddModel projectAddModel = projectInfoSaveReq.getProjectAddModel();
        ProjectInfo projectInfo = ProjectInfo.saveModelBuild(projectAddModel, SecurityUtils::getUsername);
        boolean save = this.save(projectInfo);
        if (save){
            // 属性值
            List<AttrValueModel> attrValueList = projectInfoSaveReq.getAttrValueList();
            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel, projectInfo::getId))
                    .toList();
            if (!asProductAttributeInfoList.isEmpty()){
                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
            }
            // sku ProductSkuModel -》 ProjectSkuInfo
            List<ProductSkuModel> productSkuModelList = projectInfoSaveReq.getProductSkuList();
            List<ProjectSkuInfo> projectSkuInfoList = productSkuModelList.stream().map(
                    productSkuModel -> ProjectSkuInfo.productSkuModelBuild(productSkuModel, projectInfo::getId)
            ).toList();
            if (!projectSkuInfoList.isEmpty()){
                projectSkuInfoService.saveBatch(projectSkuInfoList);
            }
        }
        return save;
    }

    /**
     * 通过商品ID获取商品详情
     *
     * @param id 商品ID
     * @return 商品详情
     */
    @Override
    public ProjectDetailResp getDetailInfo (Long id) {
        // 商品信息获取
        ProjectInfo projectInfo = this.getById(id);
        // 品牌信息
        BrandInfo brandInfo = this.brandInfoService.getById(projectInfo.getBrandId());
        // 品类集合
        List<CategoryInfo> categoryInfoList = categoryInfoService.listByIds(new ArrayList<>() {{
            add(projectInfo.getMianType());
            add(projectInfo.getParentType());
            add(projectInfo.getType());
        }});
        // 商品Sku集合
        List<ProjectSkuInfo> projectSkuInfoList = this.projectSkuInfoService.list(new LambdaQueryWrapper<>() {{
            eq(ProjectSkuInfo::getProjectId, id);
        }});

        // 商品和属性集合
        List<AsProductAttributeInfo> productAttributeInfoList = this.asProductAttributeInfoService.list(new LambdaQueryWrapper<>() {{
            eq(AsProductAttributeInfo::getProductId, id);
        }});
        // 商品规格
        List<RuleAttrAddModel> ruleAttrModelList = ruleAttrInfoService.list(new LambdaQueryWrapper<>() {{
                    eq(RuleAttrInfo::getRuleId, projectInfo.getRuleId());
                }}).stream()
                .map(RuleAttrAddModel::infoBuild).toList();


        CategoryCommonElementResp templateAttribute = this.categoryInfoService.getTemplateAttributeByCateGoryId(projectInfo.getType());
        List<TemplateAttributeGroupModel> templateAttributeGroupList = templateAttribute.getTemplateAttributeGroupList();
        List<TemplateAttributeModel> templateAttributeList = new ArrayList<>(){{
            addAll(templateAttribute.getTemplateAttributeList());
        }};
        // 属性组和商品属性的ID
        List<Long> notInAttributeIdList = new ArrayList<>();
        List<Long> attributeGroupIdList = templateAttributeGroupList.stream()
                .flatMap(templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                .map(TemplateAttributeModel::getId)
                .toList();
        List<Long> attributeIdList = templateAttributeList.stream()
                .map(TemplateAttributeModel::getId)
                .toList();
        if (!attributeGroupIdList.isEmpty()){
            notInAttributeIdList.addAll( attributeGroupIdList );
        }
        if (!attributeIdList.isEmpty()){
            notInAttributeIdList.addAll( attributeIdList );
        }
        // 添加上，商品的自有属性
        List<AsProductAttributeInfo> productAttributeList = this.asProductAttributeInfoService.list(
                new LambdaQueryWrapper<>() {{
                    eq(AsProductAttributeInfo::getProductId, projectInfo.getId());
                    notIn(AsProductAttributeInfo::getAttributeId, notInAttributeIdList);
                }}
        );

        List<TemplateAttributeModel> projectAttributeList = new ArrayList<>();
        if (!productAttributeList.isEmpty()){
            List<Long> attrIdList = productAttributeList.stream()
                    .map(AsProductAttributeInfo::getAttributeId)
                    .toList();
            projectAttributeList = attributeInfoService.list(
                                            new LambdaQueryWrapper<>() {{
                                                in(AttributeInfo::getId, attrIdList);
                                            }}
                                        ).stream()
                                        .map(TemplateAttributeModel::attributeInfoBuild)
                                        .toList();
        }
        // 把自有属性添加到商品属性的集合当中，进行合并
        if (!projectAttributeList.isEmpty()){
            templateAttributeList.addAll(projectAttributeList);
        }
        return ProjectDetailResp.builder()
                .projectInfo(projectInfo)
                .brandInfo(brandInfo)
                .categoryInfoList(categoryInfoList)
                .projectSkuInfoList(projectSkuInfoList)
                .productAttributeInfoList(productAttributeInfoList)
                .ruleAttrModelList(ruleAttrModelList)
                .attributeInfoList(templateAttributeList)
                .attributeGroupList(templateAttributeGroupList)
                .build();
    }

    /**
     * 修改商品信息
     * @param id
     * @param projectInfoEditReq
     * @return
     */
    @Override
    public Result updProject(Long id, ProjectInfoEditReq projectInfoEditReq) {
        String name = projectInfoEditReq.getName();
        //根据name查询project_info数据库
        ProjectInfo getProjectInfo=projectInfoMapper.getProjectName(name);
        if (getProjectInfo!=null){
            return Result.error("商品名称重复，不能修改");
        }
       this.updateById(ProjectAddModel.editBuild(id,projectInfoEditReq));
        if(!CollectionUtils.isEmpty(projectInfoEditReq.getAttrValueList())){
            List<AttributeInfo> attributeInfos = projectInfoEditReq.getAttrValueList().stream()
                    .filter(projectModel->!StringUtils.isBlank(projectModel.getValue()))
                    .map(item -> {
                        AttributeInfo attributeInfo = AttributeInfo.builder()
                                .id(item.getId())
                                .name(item.getValue())
                                .build();
                        return attributeInfo;
                    }).toList();
            //批量修改
            attributeInfoService.updateBatchById(attributeInfos);
        }

        if(!CollectionUtils.isEmpty(projectInfoEditReq.getProductSkuList())){
            List<ProjectSkuInfo> skuInfoList = projectInfoEditReq.getProductSkuList().stream()
                    .map(projectSkuInfoModel -> {
                        ProjectSkuInfo projectSkuInfo = ProjectSkuInfo.builder()
                                .sku(projectSkuInfoModel.getSku())
                                .image(projectSkuInfoModel.getImage())
                                .stock(projectSkuInfoModel.getStock())
                                .price(projectSkuInfoModel.getPrice())
                                .build();
                        return projectSkuInfo;
                    }).toList();
            skuInfoList.forEach(projectSkuInfoModel -> {
                LambdaUpdateWrapper<ProjectSkuInfo> wrapper = new LambdaUpdateWrapper<>();
                if(!StringUtils.isBlank(projectSkuInfoModel.getImage())){
                    wrapper.set(ProjectSkuInfo::getImage,projectSkuInfoModel.getImage());
                }
                wrapper.set(ProjectSkuInfo::getStock,projectSkuInfoModel.getStock());
                wrapper.set(ProjectSkuInfo::getPrice,projectSkuInfoModel.getPrice());
                wrapper.eq(ProjectSkuInfo::getSku, projectSkuInfoModel.getSku());
                projectSkuInfoService.update(null,wrapper);
            });


        }
        return Result.success(null,"修改成功");
    }

    @Override
    public List<ProjectInfoListResp> pageInfo(ProjectInfo projectInfo) {
        List<ProjectInfo> list = list(projectInfo);
        LinkedList<ProjectInfoListResp> listRespLinkedList = new LinkedList<>();
        list.forEach(item->{
            CategoryInfo mainTypeName = categoryInfoService.getById(item.getMianType());
            CategoryInfo parentTypeName = categoryInfoService.getById(item.getParentType());
            CategoryInfo typeName = item.getType() != null ? categoryInfoService.getById(item.getType()) : null;

            //获取规则
            RuleInfo ruleName=ruleInfoMapper.getById(item.getRuleId());
            //获取品牌信息
            BrandInfo brandName=brandInfoMapper.getById(item.getBrandId());

            //构建响应对象
            ProjectInfoListResp projectInfoListResp=ProjectInfoListResp.builder()
                   .id(item.getId())
                   .name(item.getName())
                   .introduction(item.getIntroduction())
                   .mianType(item.getMianType())
                   .parentType(item.getParentType())
                   .type(item.getType())
                   .image(item.getImage())
                   .carouselImages(item.getCarouselImages())
                   .status(item.getStatus())
                   .mianTypeName(mainTypeName!=null?mainTypeName.getName():"")
                   .parentTypeName(parentTypeName!=null?parentTypeName.getName():"")
                   .typeName(typeName!=null?typeName.getName():"")
                   .ruleName(ruleName != null ? ruleName.getName() : null)
                   .brandName(brandName != null ? brandName.getNam() : null)
                   .build();
            //添加到结果列表
            listRespLinkedList.add(projectInfoListResp);
        });

        return listRespLinkedList;
    }

//    @Override
//    public ProjectInfo getProjectName(String name) {
//        return projectInfoMapper.getProjectName(name);
//    }
}
